using ERI_Client.V1;

// ReSharper disable InconsistentNaming
namespace AIStudio.Settings.DataModel;

/// <summary>
/// An external data source, accessed via an ERI server, cf. https://github.com/MindWorkAI/ERI.
/// </summary>
public readonly record struct DataSourceERI_V1 : IExternalDataSource
{
    public DataSourceERI_V1()
    {
    }
    
    /// <inheritdoc />
    public uint Num { get; init; }

    /// <inheritdoc />
    public string Id { get; init; } = Guid.Empty.ToString();
    
    /// <inheritdoc />
    public string Name { get; init; } = string.Empty;
    
    /// <inheritdoc />
    public DataSourceType Type { get; init; } = DataSourceType.NONE;
    
    /// <summary>
    /// The hostname of the ERI server.
    /// </summary>
    public string Hostname { get; init; } = string.Empty;
    
    /// <summary>
    /// The port of the ERI server.
    /// </summary>
    public int Port { get; init; }

    /// <summary>
    /// The authentication method to use.
    /// </summary>
    public AuthMethod AuthMethod { get; init; } = AuthMethod.NONE;
}